# SPDX-FileCopyrightText: 2024-2025 MTS PJSC
# SPDX-License-Identifier: Apache-2.0
from data_rentgen.server.middlewares.monitoring.metrics import (
    apply_monitoring_metrics_middleware,
)

__all__ = ["apply_monitoring_metrics_middleware"]
